CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC                  Mongo Interactive Graphics Software                    CC
CC                 Copyright (c) 1987, 1994 - John Tonry.                  CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
* Routines to fit and evaluate Legendre polynomials
      subroutine mgoplotpoly(x,y,err,nxy,iorder,iplotit,iex)
C Fits a polynomial of order IORDER (-1 for no fit), stores the results
C in the user variables, and plots the curve (if mod(IPLOTIT,10) = 1).
C Exclude ERR=0 entries if IEX = 1
      parameter (maxpoly=8, nseg=100)
      real x(nxy), y(nxy), err(nxy)
      real scale(2)
      real*8 coeff(maxpoly)
C      REAL*8 PCOEFF(MAXPOLY)
      character*20 f
      character*150 line
      include 'mongo.par'
      include 'luns.par'
      real mgolpoly

* Plotting only?
      if(iorder.eq.-1) goto 20

* We don't allow 15th order polynomials on this bus
      if(iorder.gt.maxpoly-1) then
         write(line,'(i2)') maxpoly
         call mgoerr2('Poly order must be less than', line)
         return
      end if

* Fit the points
      npt = nxy
      ncoeff = iorder + 1
      iret = mgofitlpoly(npt,x,y,err,iex,x1,x2,y1,y2,scale,ncoeff,coeff)

* Did we have an accident?
      if(iret.eq.1) then
         write(line,'(2i7)') npt, ncoeff
         call mgoerr2('Too few points for poly fit: npt, ncoeff =',line)
         return
      else if(iret.eq.2) then
         call mgoerr('No range in x for poly fit')
         return
      else if(iret.eq.3) then
         call mgoerr('Singular equations for poly fit')
         return
      end if

* Convert Legendre coefficients to plain polynomial coefficients
C      WRITE(6,*)
C      WRITE(6,*) 'SCALE =', SCALE
C      WRITE(6,*) 'COEFFICIENTS'
C      WRITE(6,'(1P8G12.4)') (COEFF(I),I=1,NCOEFF)
C      CALL MGOPOLYC(NCOEFF,SCALE,COEFF,PCOEFF)
C      WRITE(6,'(1P8G12.4)') (PCOEFF(I),I=1,NCOEFF)
C      YPLUS = 0
C      YMINUS = 0
C      DO 666 I = NCOEFF,1,-1
C         YPLUS = PCOEFF(I) + 1D0*YPLUS
C         YMINUS = PCOEFF(I) - 1D0*YMINUS
C 666  CONTINUE
C      WRITE(6,*) MGOLPOLY(-1.0,SCALE,NCOEFF,COEFF), 
C     $     MGOLPOLY( 0.0,SCALE,NCOEFF,COEFF), 
C     $     MGOLPOLY(+1.0,SCALE,NCOEFF,COEFF)
C      WRITE(6,*) YMINUS, PCOEFF(1), YPLUS

      call mgopolyc(ncoeff,scale,coeff,coeff)
* Save the results in the user variables
      uservar(12) = iorder
      do 5 i = 1,ncoeff
         uservar(i+12) = sngl(coeff(i))
 5    continue

* Evaluate the rms of the visible (contributing) points
      rms = 0
      ncontrib = 0
      do 10 k = 1,nxy
         if(x1.lt.x2 .and. (x(k).lt.x1.or.x(k).gt.x2)) goto 10
         if(x1.gt.x2 .and. (x(k).gt.x1.or.x(k).lt.x2)) goto 10
         if(y1.lt.y2 .and. (y(k).lt.y1.or.y(k).gt.y2)) goto 10
         if(y1.gt.y2 .and. (y(k).gt.y1.or.y(k).lt.y2)) goto 10
         if(iex.eq.1.and.err(k).le.0) goto 10
         ypoly = uservar(ncoeff+12)
         do 11 i = iorder,1,-1
            ypoly = uservar(i+12) + x(k)*ypoly 
 11      continue
         rms = rms + (y(k)-ypoly)*(y(k)-ypoly)
         ncontrib = ncontrib + 1
 10   continue
      uservar(11) = sqrt(rms/max(1,ncontrib))

* Now print and plot the results (if desired)
      iord = nint(uservar(12))
 20   if(iplotit/10.eq.0) goto 30

      line = 'y = '
      nline = 4
      do 21 i = 0,iord
         call mgofpfmt(uservar(i+13),4,nch,f)
         if(i.eq.0) then
            line(nline+1:) = f(:nch)
            nline = nline + nch
         else if(i.eq.1) then
            line(nline+1:) = ' + ' // f(:nch) // ' x'
            nline = nline + nch + 5
         else
            line(nline+1:) = ' + ' // f(:nch) // ' x^' // char(i+48)
            nline = nline + nch + 7
         end if
 21   continue
      call mgofpfmt(uservar(11),3,nch,f)
      line(nline+1:) = ',  rms = ' // f(:nch)
      nline = nline + nch + 9
      call mgout(line)

 30   if(mod(iplotit,10).eq.0) return

      do 31 k = 1,nseg
         xx = x1 + float(k-1)*(x2-x1)/(nseg-1)
         ypoly = uservar(ncoeff+12)
         do 32 i = iord,1,-1
            ypoly = uservar(i+12) + xx*ypoly 
 32         continue
         if(k.eq.1) then
            call mgorelocate(xx,ypoly)
         else
            call mgodraw(xx,ypoly)
         end if
 31   continue
      return
      end

      function mgofitlpoly(npt,x,y,e,iex,x1,x2,y1,y2,scale,ncoeff,coeff)
* X and Y are data arrays with NPT points.
* SCALE returns A and B where Z = A * (X - B)
* A Legendre polynomial of degree NCOEFF-1 is fitted and 
* the coefficients are returned in COEFF:  y = Pn(z).
      parameter (maxfit=8)
      real*8 cov(maxfit*maxfit), vec(maxfit)
      real*8 coeff(1), factor
      real*8 a, b, z, mgolpk
      real x(npt), y(npt), scale(2), e(npt)
      
      n = 0
      do 1 k = 1,npt
         if(x(k).lt.amin1(x1,x2).or.x(k).gt.amax1(x1,x2)) goto 1
         if(y(k).lt.amin1(y1,y2).or.y(k).gt.amax1(y1,y2)) goto 1
         if(iex.eq.1.and.e(k).le.0) goto 1
         n = n + 1
         if(n.eq.1) then
            xmax = x(k)
            xmin = x(k)
            ymax = y(k)
            ymin = y(k)
         end if
         xmax = amax1(x(k),xmax)
         xmin = amin1(x(k),xmin)
         ymax = amax1(y(k),ymax)
         ymin = amin1(y(k),ymin)
 1    continue
      if(n.lt.ncoeff) then
         mgofitlpoly = 1
         npt = n
         return
      end if
      if(xmax.eq.xmin) then
         mgofitlpoly = 2
         return
      end if

      a = 2.0d0 / (xmax-xmin)
      b = 0.5d0 * (xmax+xmin)
      scale(1) = a
      scale(2) = b
      
      do 2 j = 1,ncoeff*ncoeff
         if(j.le.ncoeff) vec(j) = 0
         cov(j) = 0
 2    continue

      do 10 k = 1,npt
         if(x(k).lt.amin1(x1,x2).or.x(k).gt.amax1(x1,x2)) goto 10
         if(y(k).lt.amin1(y1,y2).or.y(k).gt.amax1(y1,y2)) goto 10
         if(iex.eq.1.and.e(k).eq.0) goto 10
         z = a * (dble(x(k)) - b)
         do 11 j = 1,ncoeff
            coeff(j) = mgolpk(j,z)
            if(iex.eq.1) then
               vec(j) = vec(j) + coeff(j) * y(k) / (e(k)*e(k))
            else
               vec(j) = vec(j) + coeff(j) * y(k)
            end if
            do 12 i = 1,j
               if(iex.eq.1) then
                  cov((j-1)*ncoeff+i) = cov((j-1)*ncoeff+i) + 
     $                 coeff(i) * coeff(j) / (e(k)*e(k))
               else
                  cov((j-1)*ncoeff+i) = cov((j-1)*ncoeff+i) + 
     $                 coeff(i) * coeff(j)
               end if
 12         continue
 11      continue
 10   continue

      do 13 j = 2,ncoeff
         do 14 i = 1,j-1
            cov((i-1)*ncoeff+j) = cov((j-1)*ncoeff+i)
 14      continue
 13   continue

* Wipe out lower triangular matrix entries with Gaussian elimination
      do 20 j = 1,ncoeff
         if(cov((j-1)*ncoeff+j).eq.0) then
C            write(6,*) 'Whoa there: singular matrix in fitlpoly!'
            mgofitlpoly = 3
            return
         end if
         do 21 i = j+1,ncoeff
            factor = cov((j-1)*ncoeff+i) / cov((j-1)*ncoeff+j)
            vec(i) = vec(i) - factor*vec(j)
            do 22 k = j+1,ncoeff
               cov((k-1)*ncoeff+i) = cov((k-1)*ncoeff+i) - 
     $              factor*cov((k-1)*ncoeff+j)
 22         continue
 21      continue
 20   continue

* Back substitute to solve for the coefficients
      do 30 j = ncoeff,1,-1
         do 31 i = j+1,ncoeff
            vec(j) = vec(j) - coeff(i)*cov((j-1)*ncoeff+i)
 31      continue
         coeff(j) = vec(j) / cov((j-1)*ncoeff+j)
 30   continue
      npt = n
      mgofitlpoly = 0
      
      return
      end

      real function mgolpoly(x,scale,ncoeff,coeff)
      real*8 coeff(1)
      real*8 temp, z, mgolpk
      real scale(2)
      
      z = scale(1) * (x - scale(2))
      temp = 0
      do 10 i = 1,ncoeff
         write(6,'(i4,3f12.6)') i, z, coeff(i), mgolpk(i,z)
         temp = temp + coeff(i) * mgolpk(i,z)
 10   continue
      mgolpoly = temp
      return
      end

      function mgolpk(k,z)
* Legendre polynomial of order k-1
      parameter (maxpoly=8)
      logical firstime
      real*8 mgolpk, lcoeff(maxpoly,maxpoly), c1, c2, z
      common /mgolegendre/ lcoeff
      data firstime /.true./

      if(firstime) then
         firstime = .false.
         do 3 n = 1,maxpoly
            do 4 i = 1,maxpoly
               if(i.gt.n) then
                  lcoeff(i,n) = 0
               else
                  if(n.eq.1) then
                     lcoeff(i,n) = 1d0
                  else
                     c1 = 0
                     if(i.gt.1) c1 = lcoeff(i-1,n-1)
                     c2 = 0
                     if(n.gt.2) c2 = lcoeff(i,n-2)
                     lcoeff(i,n) = ((2*n-3)*c1 - (n-2)*c2) / (n-1)
                  end if
               end if
 4          continue
 3       continue
C      write(6,*) 'Legendre coefficients: '
C      DO 100 I = 1,MAXPOLY
C         WRITE(6,'(10F12.6)') (LCOEFF(J,I),J=1,MAXPOLY)
C 100  CONTINUE

      end if

      mgolpk = lcoeff(k,k)
      do 10 i = k-2,1,-2
         mgolpk = lcoeff(i,k) + z*z*mgolpk
 10   continue
      if(2*(k/2) .eq. k) mgolpk = z*mgolpk
      return
      end

      subroutine mgopolyc(nc,scale,coeff,pcoeff)
* Routine to convert coefficients of a 1-d Legendre polynomial to
* coefficients of a plain polynomial
* The NC coefficients are stored with the low order coefficient first
      real*8 coeff(1), pcoeff(1), sc(2), temp
      real scale(2)
      parameter (maxpoly=8)
      real*8 lcoeff(maxpoly,maxpoly)
      common /mgolegendre/ lcoeff

      sc(1) = dble(scale(1))
      sc(2) = dble(scale(2))
      do 900 k = 0,nc-1
          temp = 0
          do 901 i = k,nc-1
              temp = temp + coeff(i+1) * lcoeff(k+1,i+1)
901       continue
          pcoeff(k+1) = temp
900   continue
      do 902 k = 0,nc-1
          if(sc(2).ne.0) then
             temp = 0
             do 903 i = k,nc-1
                temp = temp + pcoeff(i+1) * sc(1)**i *
     $               (-sc(2))**(i-k) * dfloat(mgobc(k,i))
 903         continue
             pcoeff(k+1) = temp
          else
             pcoeff(k+1) = pcoeff(k+1) * sc(1)**k
          end if
902   continue

      return
      end

      function mgobc(m,n)
* Returns the (m n) binomial coefficient
      mgobc = 1
      do 900 i = m+1,n
          mgobc = mgobc * i
900   continue
      do 901 i = 2,n-m
          mgobc = mgobc / i
901   continue
      return
      end
