CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC                  Mongo Interactive Graphics Software                    CC
CC                 Copyright (c) 1987, 1994 - John Tonry.                  CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
* Program to send a MONGO vector file to a PostScript laser printer
* Reads two lines: (1) Input vector file and (2) Output file name
* but writes to standard output.
      parameter (nin=2048, nbuf=1024*1024, laserwidth=2550)
      parameter (maxpath=1024)
      parameter (maxpalette=32)
      integer*2 buf(nbuf)
      character*100 outname
      character*1024 line
      integer*2 palette(3,maxpalette)
      logical eps
      data iunit /6/

1000  format(a)
1001  format(2i5,1x,a)
1002  format(i4,1x,a)
1003  format(1x,a,i1)
1004  format(3i4,1x,a)

******************************************************************************
* Vector file contains pairs of 16 bit numbers according to:                 *
*    Sign 1   Sign 2   Mantissa 1   Mantissa 2   Meaning                     *
*      0        0          x            y        Plot a dot at (x,y)         *
*      1        0          x            y        Relocate at (x,y)           *
*      0        1          x            y        Draw to (x,y)               *
*      1        1        32767        32767      End of vector file          *
*      1        1          0            0        Finish plot, start new page *
*      1        1          1            n        Line weight = n             *
*      1        1          2            n        Line type = n               *
*      1        1          3            n        Color = n (from palette)    *
*      1        1          4            n        Color (red) = n (0-256)     *
*      1        1          5            n        Color (green) = n (0-256)   *
*      1        1          6            n        Color (blue) = n (0-256)    *
*      1        1          7            n        Pointer to string (not used)*
*      1        1          8            n        Fill mode = pattern n%1024  *
*                                                     + relocmode n/1024     *
*                                                     + xy_swap n/2048       *
*      1        1          9            n        Begin square bitmap, 5xn:   *
*                                                x1, x2, y1, y2, ly clip box *
*      1        1         10            n        Begin line bitmap, 5xn:     *
*                                                x1, x2, y1, y2, ly clip box *
*      1        1         11            n        2xn locates bitmap (x0, y0) *
*      1        1         12            n        work on 16x16 square        *
*                                                then n = 4 pixels of bitmap *
*                                                p1+ 10*p2+ 100*p3+ 1000*p4, *
*      1        1         13            n        work on line                *
*                                                then n = 6 bits of bitmap   *
*      1        1         14            n        End bitmap                  *
* (x,y) are oriented in portrait mode                                        *
******************************************************************************

      ITEST = 0

* Read list of vectors and find how many there are
      read(5,1000) line
      open (unit=1,file=line,form='unformatted',status='old')
      ncoord = 0
      do 1 j = 1,nbuf/nin
          read(1,end=2) (buf(i+ncoord),i=1,nin)
          ncoord = ncoord + nin
1     continue
2     close(1)

* Read output file name
      read(5,1000) outname

* Is there a fill command in this?  How about bounding box?
      megotafill = 0
      megotabitmap = 0
      megotalinemap = 0
      minx = 1000000000
      maxx = -1
      miny = 1000000000
      maxy = -1
      i = 1
 3    if(i.lt.ncoord) then
         j = buf(i) + 32768
         k = buf(i+1) + 32768
         if(buf(i+1).gt.0.or.buf(i).gt.0) then
            ix = buf(i)
            if(ix.lt.0) ix = ix + 32768
            iy = buf(i+1)
            if(iy.lt.0) iy = iy + 32768
            minx = min(minx, ix)
            maxx = max(maxx, ix)
            miny = min(miny, iy)
            maxy = max(maxy, iy)
         else
            if(j .eq. 8) megotafill = 1
            if(j .eq. 9) megotabitmap = 1
            if(j .eq.10) megotalinemap = 1
            if(j .eq. 9 .or. j.eq.10) then
               m2 = buf(i+1)
               if(m2.lt.0) m2 = m2 + 32768
               minx = min(minx, m2)
               m2 = buf(i+3)
               if(m2.lt.0) m2 = m2 + 32768
               maxx = max(maxx, m2)
               m2 = buf(i+5)
               if(m2.lt.0) m2 = m2 + 32768
               miny = min(miny, m2)
               m2 = buf(i+7)
               if(m2.lt.0) m2 = m2 + 32768
               maxy = max(maxy, m2)
               i = i + 8
            end if
            if(j .eq. 32767) ncoord = i+1
         end if
         i = i + 2
         goto 3
      end if

* Initialize the palette
      icolor = 1
      do 5 j = 1,maxpalette-1
         palette(1,j) = 0
         palette(2,j) = 0
         palette(3,j) = 0
 5    continue
      palette(1,maxpalette) = 255
      palette(2,maxpalette) = 255
      palette(3,maxpalette) = 255
      ifillstart = 0
      ifillmode = 0
      irelocmode = 0
      ifilling = 0
      nbitmap = 0

* Initialize the laser printer
      line(:100) = outname
      call lower(line(:100))
      eps = index(line,'.eps') .gt. 0
      call psheader(iunit,minx,miny,maxx,maxy,eps,outname)
      if(megotafill.eq.1) call fillheader(iunit)
      if(megotabitmap.eq.1) call bitsqheader(iunit)
      if(megotalinemap.eq.1) call bitlineheader(iunit)
      call pstrailer(iunit,eps)

* Run through the vectors, sending them to the printer
* Note that several passes may be necessary for multipage plots
      if(eps) then
         mxpx = minx
         my = miny
      else
         mxpx = 0
         my = 0
      end if
20    continue
      mx = mxpx
      mxpx = mxpx + laserwidth
      n = 0
      more = 0
      ndraw = 0
      do 10 i = 1,ncoord,2
          i1 = 0
          m1 = buf(i)
          if(m1 .lt. 0) then
             i1 = 1
             m1 = m1 + 32768
          end if
          i2 = 0
          m2 = buf(i+1)
          if(m2 .lt. 0) then
             i2 = 1
             m2 = m2 + 32768
          end if

          mode = i1 + 2*i2

          IF(ITEST.EQ.1) THEN
             WRITE(3,'(10I6)') I, I1, M1, I2, M2, MODE, 
     $            IFILLMODE, IRELOCMODE, IFILLING
          END IF

* Test to see whether the vector is on the present page
          if(i1.eq.0.or.i2.eq.0) then
              if(m1.gt.mxpx) then
                  more = 1
                  goto 10
              else if(m1.lt.mx) then
                  goto 10
              end if
          end if
* Write the output line if it will exceed the length allotted
          if(n.gt.(80-12).and.nbitmap.eq.0) then
              write(iunit,1000) line(:n)
              n = 0
          end if

* trigger any pending fills (but not on draw or reloc [with reloc mode])
          if(ifilling.eq.1) then
             if(mode.ne.2 .and. (mode.ne.1.or.irelocmode.eq.0)) then
                if(n.gt.0) write(iunit,1000) line(:n)
                n = 0
                call fillarea(ifillstart, i-1, mx,my, buf, 
     $               ifillmode, iunit)
                ifilling = 0
                IF(ITEST.EQ.1) THEN
                   WRITE(3,'(I6,A,10I6)') I, ' FILL: ', IFILLSTART, I-1
                END IF

             end if
          end if

* point or draw?
          if(i1.eq.0) then

* point at m1, m2
              if(i2.eq.0) then
                  write(line(n+1:n+12),1001) m1-mx, m2-my, 'p'
                  n = n + 12
                  ndraw = 0
              else

* draw to m1, m2
                  write(line(n+1:n+12),1001) m1-mx, m2-my, 'd'
                  n = n + 12
                  ndraw = ndraw + 1
* Check to see if our path is too long
                  if(ndraw.ge.maxpath) then
                      ndraw = 0
* Write the output line if it will exceed the length allotted
                      if(n.gt.(80-12)) then
                          write(iunit,1000) line(:n)
                          n = 0
                      end if
                      write(line(n+1:n+12),1001) m1-mx, m2-my, 'r'
                      n = n + 12
                  end if
              end if

* relocate or special?
          else

* relocate at m1, m2
              if(i2.eq.0) then
                  if(ifillmode.gt.0 .and. ifilling.eq.0) then
                     ifilling = 1
                     ifillstart = i
                  end if
                  write(line(n+1:n+12),1001) m1-mx, m2-my, 'r'
                  ixreloc = m1 - mx
                  iyreloc = m2 - my
                  n = n + 12
                  ndraw = 0

* end of buffer
              else if(m1.eq.32767) then
                  goto 11

* eject page, start new one
              else if(m1.eq.0) then
                  if(n.gt.0) write(iunit,1000) line(:n)
                  n = 0
                  write(iunit,1000) 'printpage'
                  ndraw = 0

* set line weight
              else if(m1.eq.1) then
                  write(line(n+1:n+7),1002) m2, 'lw'
                  n = n + 7
                  ndraw = 0

* set line type
              else if(m1.eq.2.and.m2.ge.0.and.m2.le.6) then
                  write(line(n+1:n+4),1003) 'lt',m2
                  n = n + 4
                  ndraw = 0

* Set color - from palette. (This will probably never be used).
              else if(m1.eq.3) then
                 if(n.gt.0) write(iunit,1000) line(:n)
                 n = 0
                 ndraw = 0
                 icolor = m2
                 write(iunit,1004) palette(1,icolor), 
     $                palette(2,icolor),palette(3,icolor),'rgb'

* Set color - red.  Note this presumes a sequence of (4,r) (5,g) (6,b)
              else if(m1.eq.4) then
                 if(n.gt.0) write(iunit,1000) line(:n)
                 n = 0
                 ndraw = 0
                 palette(1,icolor) = min(255,max(0,m2))
* Set color - green (continuation, we trust)
              else if(m1.eq.5) then
                 palette(2,icolor) = min(255,max(0,m2))
* Set color - blue (continuation, we trust)
              else if(m1.eq.6) then
                 palette(3,icolor) = min(255,max(0,m2))
                 write(iunit,1004) palette(1,icolor), 
     $                palette(2,icolor),palette(3,icolor),'rgb'

* Strings aren't supported by Mongo for postscript
              else if(m1.eq.7) then

* Set fill mode
              else if(m1.eq.8) then
                 m = mod(m2,1024)
                 irelocmode = mod(m2/1024,2)
                 ixyswap = mod(m2/2048,2)
* Here's a hack... if XYSWAPPED some of the bit patterns will be
* rotated by 90 degrees.  But with secret information they can be
* exchanged for their neighbors with the correct orientation.
                 if(ixyswap.eq.1) then
                    if(m.eq.5 .or. m.eq.7 .or. m.eq.9 .or. m.eq.11) then
                       m = m + 1
                    else if(m.eq.6 .or. m.eq.8 .or. 
     $                      m.eq.10 .or. m.eq.12) then
                       m = m - 1
                    end if
                 end if
                 ifillmode = m

* Start a bitmap
              else if(m1.eq.9 .or. m1.eq.10) then
                 if(nbitmap.eq.0) then
                    if(n.gt.0) write(iunit,1000) line(:n)
                    n = 0
                    ndraw = 0
                    write(iunit,1000) 'gsave'
                    if(m1.eq.9) then
                       write(iunit,1000) '/GrayFont4 findfont'//
     $                      ' 1 scalefont setfont'
                    else
                       write(iunit,1000) '/GrayFont1 findfont'//
     $                      ' 1 scalefont setfont'
                    end if
                    nx1clip = m2 - mx
                 else if(nbitmap.eq.1) then
                    nx2clip = m2 - mx
                 else if(nbitmap.eq.2) then
                    ny1clip = m2 - my
                 else if(nbitmap.eq.3) then
                    ny2clip = m2 - my
                 else if(nbitmap.eq.4) then
                    idy = m2
                 end if
                 nbitmap = nbitmap + 1
                 if(nbitmap.eq.5) then
                    idxbit = nx2clip - nx1clip
                    idybit = ny2clip - ny1clip
                    if(idy.gt.0) then
                       write(iunit,1008) -idy
 1008                  format('90 rotate 0', i6, ' translate')
                    end if
                    write(iunit,1009) nx1clip, ny1clip, 
     $                   idybit, idxbit, -idybit
 1009               format(i5,i5,' moveto 0',i5,' rlineto',i5,
     $                   ' 0 rlineto 0 ',i5,' rlineto closepath clip')
                    nbitmap = 0
                 end if

* Start bitmap: next two define starting point of corner
              else if(m1.eq.11) then
                 if(nbitmap.eq.0) then
                    ixbit = m2 - mx
                    nbitmap = nbitmap + 1
                 else
                    iybit = m2 - my
                    nbitmap = 0
                 end if

* Draw a 16x16 bitmap... we trust that 64 consecutive carry this info
              else if(m1.eq.12) then
                 if(nbitmap.eq.0) then
                    n = n + 1
                    line(n:n) = '('
                    nonblank = 0
                 end if
                 k1 = mod(m2,     10) + 10*mod(m2/10,  10)
                 call bitmapchar(n,line,k1)
                 if(k1.gt.0) nonblank = nonblank + 1
                 k2 = mod(m2/100, 10) + 10*mod(m2/1000,10)
                 call bitmapchar(n,line,k2)
                 if(k2.gt.0) nonblank = nonblank + 1

                 nbitmap = nbitmap + 2
                 if(nbitmap.eq.128) then
                    n = n + 1
                    line(n:n) = ')'
                    write(line(n+1:),1001) ixbit, iybit, 'Sq'
                    n = n + 13
                    nbitmap = 0
                    if(nonblank.gt.0) then
                       write(iunit,1000) line(:n)
                    end if
                    n = 0
                 end if

* Draw a line bitmap... we trust enough subsequent carry this info
              else if(m1.eq.13) then
                 if(nbitmap.eq.0) then
                    n = n + 1
                    line(n:n) = '('
                    nonblank = 0
                 end if
                 if(m2.gt.0) nonblank = nonblank + 1
                 k1 = m2/64 + ichar('*')
                 if(k1.ge.92) k1 = k1 + 1
                 k2 = mod(m2,64) + ichar('*')
                 if(k2.ge.92) k2 = k2 + 1
                 line(n+1:n+2) = char(k1)//char(k2)
                 n = n + 2
                 nbitmap = nbitmap + 12
                 if(nbitmap.ge.idxbit) then
                    n = n + 1
                    line(n:n) = ')'
                    write(line(n+1:),1001) ixbit, iybit, 'Lb'
                    n = n + 13
                    nbitmap = 0
                    if(nonblank.gt.0) then
                       write(iunit,1000) line(:n)
                    end if
                    n = 0
                 end if

* Finish bitmap
              else if(m1.eq.14) then
                    write(iunit,1000) 'grestore'
              end if

          end if
10    continue

11    continue
      if(more.eq.0) goto 999
      if(n.gt.0) write(iunit,1000) line(:n)
      write(iunit,1000) 'printpage'
      goto 20

999   continue
* Write the LINE buffer if it is not empty and print the page
      if(n.gt.0) write(iunit,1000) line(:n)
      write(iunit,1000) 'printpage'

* Finish up the plot
      write(iunit,1000) '%%Trailer'
      write(iunit,1000) 'EndPSPage'
      write(iunit,1000) 'end restore'
      write(iunit,1000) '%%EOF'

      stop
      end

      subroutine fillarea(i1, i2, mx, my, buf, ifillmode, iunit)
      integer*2 buf(1)
      character*80 line
 1000 format(a)
 1001 format(2i5,a)
 1003 format(1x,a)
 1002 format(1x,'pat',a1,i4,a)

* By how much is the original 8x8 pattern expanded when sent to printer?
      iexpand = 2

      n = 0
      do 10 i = i1, i2, 2
         m1 = buf(i)
         if(m1.lt.0) m1 = m1 + 32768
         m2 = buf(i+1)
         if(m2.lt.0) m2 = m2 + 32768
         if(i.eq.i1) then
            write(line(n+1:),1001) m1-mx, m2-my, ' r'
            n = n + 12
         else
            write(line(n+1:),1001) m1-mx, m2-my, ' d'
            n = n + 12
         end if
         if(n.gt.80-12) then
            write(iunit,1000) line(:n)
            n = 0
         end if
 10   continue
      if(ifillmode.gt.1) then
         iasc = ifillmode - 2 + ichar('a')
         write(line(n+1:),1002) char(iasc), 8*iexpand, ' fp'
         n = n + 12
      else
         write(line(n+1:),1000) ' f'
         n = n + 2
      end if
      write(iunit,1000) line(:n)
      return
      end

      function nch(s)
      character*(*) s
      do 900 i = len(s),1,-1
          nch = i
          if(s(i:i).ne.' '.and.ichar(s(i:i)).ne.0) return
900   continue
      return
      end

      subroutine lower(s)
      character*(*) s
      do 900 i = 1,len(s)
         if(s(i:i).ge.'A'.and.s(i:i).le.'Z') then
            s(i:i) = char(ichar(s(i:i)) + 32)
         end if
900   continue
      return
      end

      subroutine psheader(iunit,minx,miny,maxx,maxy,eps,outname)
      character*(*) outname
      character*100 line
      character*20 when
      logical eps
 1000 format(10a)
 1001 format(a,4i5)
      if(eps) then
         write(iunit,1000) '%!PS-Adobe-2.0 EPSF 2.0'
      else
         write(iunit,1000) '%!PS-Adobe-2.0'
      end if
      write(iunit,1000) '%%Title: ', outname(:nch(outname))
      write(iunit,1000) '%%Creator: Mongo2k Version 1.0'
      when = ' '
      call mgodate(when)
      call mgotime(when(12:))
      write(iunit,1000) '%%CreationDate: ' // when
      if(eps) then
         ix1 = 0
         iy1 = 0
         ix2 = int((maxx-minx)*72./300.+0.9999)
         iy2 = int((maxy-miny)*72./300.+0.9999)
      else
         ix1 = int(minx*72./300.)
         iy1 = int(miny*72./300.)
         ix2 = int(maxx*72./300.+0.9999)
         iy2 = int(maxy*72./300.+0.9999)
      end if
      write(iunit,1001) '%%BoundingBox:',ix1,iy1,ix2,iy2
      write(iunit,1000) '%%EndComments'
      write(iunit,1000) 'save 50 dict begin'
      write(iunit,1000) '/StartPSPage {/PSPage save def newpath',
     $     ' 0 0 moveto 0 setlinewidth 0 setgray 1 setlinecap',
     $     ' 1 setlinejoin 72 300 div dup scale} def'
      write(iunit,1000) '/pending {false} bind def'
      write(iunit,1000) '/finish {pending {currentpoint stroke ',
     $     'moveto /pending false def} if} bind def'
      write(iunit,1000) '/r {finish newpath moveto} bind def'
      write(iunit,1000) '/d {lineto /pending true def} bind def'
      write(iunit,1000) '/l {finish 4 2 roll moveto lineto ',
     $     'currentpoint stroke moveto} bind def'
      write(iunit,1000) '/p {finish newpath moveto currentpoint ',
     $     'lineto currentpoint stroke moveto} bind def'
      if(eps) then
         write(iunit,1000) '/printpage {finish newpath 0 0 moveto} def'
      else
         write(iunit,1000) '/printpage {finish showpage ',
     $        'newpath 0 0 moveto} def'
      end if
      write(iunit,1000) '/lw {finish setlinewidth} bind def'
      write(iunit,1000) '/lt0 {finish [] 0 setdash} bind def'
      write(iunit,1000) '/lt1 {finish [3 5] 0 setdash} bind def'
      write(iunit,1000) '/lt2 {finish [20 10] 0 setdash} bind def'
      write(iunit,1000) '/lt3 {finish [60 10] 0 setdash} bind def'
      write(iunit,1000) '/lt4 {finish [3 10 20 10] 0 setdash} bind def'
      write(iunit,1000) '/lt5 {finish [3 10 60 10] 0 setdash} bind def'
      write(iunit,1000) '/lt6 {finish [20 10 60 10] 0 setdash} bind def'
* Use these for more space in case fat postscript lines run together
C      write(iunit,1000) '/lt1 {finish [4 8] 0 setdash} bind def'
C      write(iunit,1000) '/lt2 {finish [25 15] 0 setdash} bind def'
C      write(iunit,1000) '/lt3 {finish [60 20] 0 setdash} bind def'
C      write(iunit,1000) '/lt4 {finish [4 15 25 15] 0 setdash} bind def'
C      write(iunit,1000) '/lt5 {finish [4 20 60 20] 0 setdash} bind def'
C      write(iunit,1000) '/lt6 {finish [20 18 60 18] 0 setdash} bind def'
      write(iunit,1000) '/rgb {finish 3 1 roll 255 div 3 1 roll',
     $     ' 255 div 3 1 roll 255 div setrgbcolor} bind def'
      return
      end

      subroutine fillheader(iunit)
 1000 format(10a)
      write(iunit,1000) '/f {gsave closepath fill grestore'//
     $     ' /pending false def} def'
      write(iunit,1000) '/fp {gsave setpattern closepath fill'//
     $     ' grestore /pending false def} def'
      write(iunit,1000) '/setpatterndict 20 dict def'
      write(iunit,1000) 'setpatterndict begin'
      write(iunit,1000) '  /tempctm matrix def'
      write(iunit,1000) '  /bitison {/ybit exch def /xbit exch def'
      write(iunit,1000) '     /bytevalue bstring 7 ybit sub get def'
      write(iunit,1000) '     /mask 1 7 xbit 8 mod sub bitshift def'
      write(iunit,1000) '     bytevalue mask and 0 ne} def'
      write(iunit,1000) 'end'
      write(iunit,1000) '/spotfunction'
      write(iunit,1000) '  {setpatterndict begin'
      write(iunit,1000) '    /y exch def /x exch def'
      write(iunit,1000) '    /xindex x 1 add 2 div 8 mul cvi def'
      write(iunit,1000) '    /yindex y 1 add 2 div 8 mul cvi def'
      write(iunit,1000) '    xindex yindex bitison'
      write(iunit,1000) '     {/onbits onbits 1 add def 1}'
      write(iunit,1000) '     {/offbits offbits 1 add def 0}  ifelse'
      write(iunit,1000) '   end  } def'
      write(iunit,1000) '/setpattern'
      write(iunit,1000) '  {setpatterndict begin'
      write(iunit,1000) '     /cellsize exch def  /bstring exch def'
      write(iunit,1000) '     /onbits 0 def /offbits 0 def'
      write(iunit,1000) '     /m tempctm currentmatrix def'
      write(iunit,1000) '     /veclength 1 0 m dtransform pop'//
     $     ' cellsize mul def'
      write(iunit,1000) '     /oneinch 72 0 tempctm defaultmatrix'//
     $     ' dtransform pop def'
      write(iunit,1000) '     /frequency oneinch veclength div def'
      write(iunit,1000) '     frequency 0 /spotfunction load setscreen'
      write(iunit,1000) '     {} settransfer'
      write(iunit,1000) '     offbits offbits onbits add div setgray'
      write(iunit,1000) '   end   } def'
      write(iunit,1000) '/pata <55aa55aa55aa55aa> def'
      write(iunit,1000) '/patb <5500550055005500> def'
      write(iunit,1000) '/patc <1100000011000000> def'
      write(iunit,1000) '/patd <1111111111111111> def'
      write(iunit,1000) '/pate <ff000000ff000000> def'
      write(iunit,1000) '/patf <0404040404040404> def'
      write(iunit,1000) '/patg <0000ff0000000000> def'
      write(iunit,1000) '/path <2244881122448811> def'
      write(iunit,1000) '/pati <4422118844221188> def'
      write(iunit,1000) '/patj <0102040810204080> def'
      write(iunit,1000) '/patk <8040201008040201> def'
      write(iunit,1000) '/patl <ff111111ff111111> def'
      write(iunit,1000) '/patm <0404ff0404040404> def'
      write(iunit,1000) '/patn <8142241818244281> def'
      write(iunit,1000) '/pato <4281182424188142> def'
      write(iunit,1000) '/patp <001c2222221c0000> def'
      write(iunit,1000) '/patq <2442811818814224> def'
      write(iunit,1000) '/patr <814e48781e127281> def'
      write(iunit,1000) '/pats <881d3e5c88c5e3d1> def'
C Fish scales, ugh
C      write(iunit,1000) '/pats <080814e38080413e> def'
      return
      end

      subroutine pstrailer(iunit,eps)
      logical eps
 1000 format(a)
      write(iunit,1000) '/EndPSPage {PSPage restore} def'
      write(iunit,1000) '%%EndProlog'
      write(iunit,1000) 'StartPSPage'
      write(iunit,1000) '%%Page: 1 1'
      return
      end

      subroutine bitmapchar(n,line,k)
      character*(*) line
      if(k.le.6) then
         mapchar = k + 32
      else if(k.le.8) then
         mapchar = k - 7
      else if(k.le.9) then
         mapchar = k + 30
      else if(k.le.17) then
         mapchar = k + 32
      else if(k.le.19) then
         mapchar = k - 16
      else if(k.le.61) then
         mapchar = k + 30
      else if(k.le.79) then
         mapchar = k + 31
      else if(k.le.81) then
         mapchar = k - 76
      else if(k.le.90) then
         mapchar = k + 29
      else if(k.le.92) then
         mapchar = k - 85
      else
         mapchar = k + 27
      end if

      if(line(n:n).eq.'(') lastoctal = 0

      n = n + 1
      if(mapchar.lt.32) then
         line(n:n+1) = char(92)//char(mapchar+ichar('0'))
         n = n + 1
         lastoctal = 1
      else
* Pad out a previous octal number if this char will cause ambiguity
         if(lastoctal.eq.1 .and.
     $        mapchar.ge.ichar('0').and.mapchar.le.ichar('7')) then
            line(n+1:n+1) = line(n-1:n-1)
            line(n-1:n) = '00'
            n = n + 2
         end if
         line(n:n) = char(mapchar)
         lastoctal = 0
      end if

      return
      end

      subroutine bitsqheader(iunit)
 1000 format(a)
      write(iunit,1000) '/newfonta 10 dict def newfonta begin'
      write(iunit,1000) '/FontType 3 def /FontMatrix'//
     $     ' [1 0 0 1 0 0] def /FontBBox [0 0 8 4] def'
      write(iunit,1000) '/Encoding 256 array def'
      write(iunit,1000) '0 1 255 {Encoding exch /.notdef put} for'
      write(iunit,1000) '/CharData 110 dict def CharData begin'
      write(iunit,1000) '/newname 2 string def'
      write(iunit,1000) '/grayfont [ <00000000> <00020000>'//
     $     ' <00020400> <00060600>'
      write(iunit,1000) '<08020401> <08060601> <09060609>'//
     $     ' <0907060b> <090f0f0b> <0f0f0f0f> ] def'
      write(iunit,1000) '/.notdef {<00000000>} def'
      write(iunit,1000) '/nfont grayfont length def'
      write(iunit,1000) '0 1 nfont nfont mul 1 sub {dup'
      write(iunit,1000) ' /char exch  dup  6 le {32 add} {dup  8 le '//
     $     '{ 7 sub} {dup  9 eq {30 add} '
      write(iunit,1000) ' {dup 17 le {32 add} {dup 19 le {16 sub} '//
     $     '{dup 61 le {30 add} '
      write(iunit,1000) ' {dup 79 le {31 add} {dup 81 le {76 sub} '//
     $     '{dup 90 le {29 add} '
      write(iunit,1000) ' {dup 92 le {85 sub} {27 add} ifelse } '//
     $     'ifelse } ifelse } ifelse }'
      write(iunit,1000) ' ifelse } ifelse } ifelse } ifelse } '//
     $     'ifelse } ifelse def'
C      write(iunit,1000) '  /char exch 42 add dup 92 ge {1 add} if def'
      write(iunit,1000) '  dup nfont idiv /first exch def'
      write(iunit,1000) '     nfont mod  /second exch def'
      write(iunit,1000) '  newname 0 first 65 add put'
      write(iunit,1000) '  newname 1 second 65 add put'
      write(iunit,1000) '  Encoding char newname cvn put'
      write(iunit,1000) '  newname cvn 4 string def'
      write(iunit,1000) '0 1 3 {newname cvn cvx exec exch dup'
      write(iunit,1000) '  dup grayfont first get exch get'
      write(iunit,1000) '  exch grayfont second get exch get'//
     $     ' 4 bitshift add'
      write(iunit,1000) '  put} for } for    end'
      write(iunit,1000) '/BuildChar { 8 0 0 0 8 4 setcachedevice exch '
      write(iunit,1000) 'begin Encoding exch get CharData exch get'
      write(iunit,1000) '8 exch 4 exch true exch [1 0 0 1 0 0]'//
     $     ' exch imagemask end } def end'
      write(iunit,1000) '/GrayFont4 newfonta definefont pop'
      write(iunit,1000) '/Sq { /y exch def /x exch def'//
     $     ' /text exch def 0 16 255 { x y moveto '
      write(iunit,1000) '       2 div cvi text exch 8 getinterval'//
     $     ' show /y y 4 add def } for  } bind def'
      return
      end

      subroutine bitlineheader(iunit)
 1000 format(a)
      write(iunit,1000) '/newfont1 10 dict def newfont1 begin'
      write(iunit,1000) '/FontType 3 def /FontMatrix'//
     $     ' [1 0 0 1 0 0] def /FontBBox [0 0 6 1] def'
      write(iunit,1000) '/Encoding 256 array def'
      write(iunit,1000) '0 1 255 {Encoding exch /.notdef put} for'
      write(iunit,1000) '/CharData 75 dict def CharData begin'
      write(iunit,1000) '/newname 2 string def'
      write(iunit,1000) '/.notdef {<00>} def'
      write(iunit,1000) '0 1 63 {dup'
      write(iunit,1000) ' /char exch  42 add dup 92 ge {1 add} if def'
      write(iunit,1000) '  dup 8 idiv /first exch def'
      write(iunit,1000) '  dup 8 mod  /second exch def'
      write(iunit,1000) '  newname 0 first 65 add put'
      write(iunit,1000) '  newname 1 second 65 add put'
      write(iunit,1000) '  Encoding char newname cvn put'
      write(iunit,1000) '  newname cvn 1 string def'
      write(iunit,1000) '  newname cvn cvx exec exch 0 exch '//
     $     '2 bitshift put } for end'
      write(iunit,1000) '/BuildChar { 6 0 0 0 6 1 setcachedevice exch '
      write(iunit,1000) 'begin Encoding exch get CharData exch get'
      write(iunit,1000) '6 exch 1 exch true exch [1 0 0 1 0 0]'//
     $     ' exch imagemask end } def end'
      write(iunit,1000) '/GrayFont1 newfont1 definefont pop'
      write(iunit,1000) '/Lb {moveto show} bind def'
      return
      end
