/*CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC                  Mongo Interactive Graphics Software                    CC
CC                 Copyright (c) 1987, 1994 - John Tonry.                  CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC*/
#include <stdio.h>
#include <math.h>

#define NFITS (2880)
#define ERR_CANT_OPEN_FILE		1 
#define ERR_CANT_READ_HEADER		2 
#define ERR_CANT_READ_DATA		3 
#define ERR_CANT_WRITE_HEADER		4 
#define ERR_CANT_WRITE_DATA		5 
#define ERR_INSUFFICIENT_HEADER		6 
#define ERR_NO_NAXIS			7 
#define ERR_NO_END			8 
#define ERR_INCONSISTENT_NAXIS		9 

static char buf[NFITS];

/* Read a short integer file's header and data */

rffile_(n,head,bscale,bzero,nx,ny,data,file,headlen,filelen)
     int *n, *nx, *ny, headlen, filelen;
     short *data;
     char *head, *file;
     float *bscale, *bzero;
{
  int err;
  err = rfitsns(16,n,head,bscale,bzero,nx,ny,data,file,headlen,filelen);
#ifdef SWAB
  swab(data,data,2*(*nx)*(*ny));
#endif
  return(err);
}

/* Just read the bits, with bitsper bits per data element */
rfitsns(bitsper,n,head,bscale,bzero,nx,ny,data,file,headlen,filelen)
     int bitsper;
     int *n, *nx, *ny, headlen, filelen;
     unsigned char *data;
     char *head, *file;
     float *bscale, *bzero;
{
  int fd, err, need, i, nread;

  while(--filelen > 0 && (file[filelen]==' ' || file[filelen]==0));
  for(i=0;i<=filelen;i++) buf[i] = file[i] ;
  buf[filelen+1] = '\0';

  if((fd=open(buf,0)) == -1) {
    fprintf(stderr,"rffile: cannot open file %s\n",buf);
    (*nx)=(*ny)=0;
    return(ERR_CANT_OPEN_FILE);
  }

/* Read the header, looking for NAXIS1, NAXIS2 and END */
  if((err = getheader(fd,n,head,bscale,bzero,nx,ny)) != 0) {
    close(fd);
    (*nx)=(*ny)=0;
    return(err);
  }

/* Read the data according to the values for NX, NY */
  need = (bitsper*(*nx)*(*ny)+7) / 8;
  if((nread=read(fd,data,need)) != need) {
    fprintf(stderr,"rffile: end of file before data read\n");
    (*nx)=(*ny)=0;
    close(fd);
    return(ERR_CANT_READ_DATA);
  }
  close(fd);
  return(0);
}

getheader(fd,n,head,bscale,bzero,nx,ny)
     int fd, *n, *nx, *ny;
     char *head;
     float *bscale, *bzero;
{
  int i, nread, extra, end;

  end = 0;
  for(i=0;i<(*n)*80;i+=80) {
    if((nread=read(fd,head+i,80)) != 80) {
      fprintf(stderr,"rffile: error reading header; END not found\n");
      return(1);
    }
    if(head[i]=='E' && head[i+1]=='N' && head[i+2]=='D') {
      end = 1;
      break;
    }
  }
  if(end == 0) {
    fprintf(stderr,"rffile: insufficient buffer for header\n");
    return(ERR_INSUFFICIENT_HEADER);
  }

  i+= 80;
  extra = NFITS - (i%NFITS);
  if((i%NFITS) !=0) {
    if((nread = read(fd,buf,extra)) != extra) {
      fprintf(stderr,"rffile: error reading header, file too short\n");
      return(ERR_CANT_READ_HEADER);
    }
  }
  *n = i / 80;
  *nx = 0;
  *ny = 0;
  *bscale = 1.0;
  *bzero = 0.0;
  for(i=0; i<(*n); i++) {
    if(strncmp(head+80*i,"NAXIS1",6) == 0) *nx = atoi(head+80*i+10);
    if(strncmp(head+80*i,"NAXIS2",6) == 0) *ny = atoi(head+80*i+10);
    if(strncmp(head+80*i,"BSCALE",6) == 0) *bscale = atof(head+80*i+10);
    if(strncmp(head+80*i,"BZERO",5) == 0) *bzero = atof(head+80*i+10);
  }
  if(*nx==0 || *ny==0) {
    fprintf(stderr,"rffile: cannot find NX, NY\n");
    return(ERR_NO_NAXIS);
  }   
  return(0);
}
